import { NextApiRequest, NextApiResponse } from 'next';

interface WithExceptionHandlingOptions {
    shouldIncludeErrorContext?: (req: NextApiRequest) => boolean;
    getAdditionalErrorContext?: (req: NextApiRequest) => Record<string, unknown>;
}

interface WithOkStatusOptions {
    addIf?: (req: NextApiRequest) => boolean;
}

interface HttpExceptionMetadata {
    type: string;
    message: string;
    data?: Record<string, unknown>;
}
interface ThrowingOptions {
    json?: boolean;
}
/**
 * Throw HttpExceptions inside API endpoints to generate nice error messages
 *
 * @example
 * ```
 * if (bad_soups.includes(soup_param)) {
 *   throw new HttpException(400, {
 *      type: "cant_make_soup",
 *      message: "Soup was too difficult, please specify a different soup",
 *      data: { soup_param }
 *    })
 * }
 * ```
 *
 **/
declare class HttpException extends Error {
    status: number;
    metadata: HttpExceptionMetadata;
    options: ThrowingOptions;
    constructor(status: number, metadata: HttpExceptionMetadata, options?: ThrowingOptions);
    toString(): string;
}
/**
 * Throw BadRequestException inside API endpoints that were provided incorrect
 * parameters or body
 *
 * @example
 * ```
 * if (bad_soups.includes(soup_param)) {
 *   throw new BadRequestException({
 *     type: "cant_make_soup",
 *     message: "Soup was too difficult, please specify a different soup",
 *   })
 * }
 * ```
 *
 **/
declare class BadRequestException extends HttpException {
    metadata: HttpExceptionMetadata;
    constructor(metadata: HttpExceptionMetadata, options?: ThrowingOptions);
}
declare class UnauthorizedException extends HttpException {
    metadata: HttpExceptionMetadata;
    constructor(metadata: HttpExceptionMetadata, options?: ThrowingOptions);
}
declare class NotFoundException extends HttpException {
    metadata: HttpExceptionMetadata;
    constructor(metadata: HttpExceptionMetadata, options?: ThrowingOptions);
}
declare class MethodNotAllowedException extends HttpException {
    metadata: HttpExceptionMetadata;
    constructor(metadata: HttpExceptionMetadata, options?: ThrowingOptions);
}
declare class InternalServerErrorException extends HttpException {
    metadata: HttpExceptionMetadata;
    constructor(metadata: HttpExceptionMetadata, options?: ThrowingOptions);
}

interface ExceptionHandlingOptions {
    addOkStatus?: boolean;
    okStatusOptions?: WithOkStatusOptions;
    exceptionHandlingOptions?: WithExceptionHandlingOptions;
}
declare const withExceptionHandling: ({ addOkStatus, okStatusOptions, exceptionHandlingOptions, }?: ExceptionHandlingOptions) => (next: (req: NextApiRequest, res: NextApiResponse) => Promise<void>) => (req: NextApiRequest, res: NextApiResponse) => Promise<void>;

export { BadRequestException, ExceptionHandlingOptions, HttpException, HttpExceptionMetadata, InternalServerErrorException, MethodNotAllowedException, NotFoundException, ThrowingOptions, UnauthorizedException, withExceptionHandling };
